/*  Copyright (C) 2001Bjrn Paetzel                     kolrabi@gmx.de
   
    Bjrn Paetzel
	Gelsenkircher Str. 66
	D-44649 Herne

  ------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation;  either  version 2 of the  License, or
    any later version.

    This program is distributed in the hope that it  will be useful, but
    WITHOUT  ANY  WARRANTY;  without   even  the  implied   warranty  of
	MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU
    General Public License for more details.
	
	You should have received a copy  of the  GNU  General Public License
    along with this program (GPL.TXT); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, 
	Boston, MA
    02111-1307, USA.
*/

// ---------------------------------------------------------- //
// clientsockets.c

// ---------------------------------------------------------- //
// includes

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#ifdef _WIN32

#include <windows.h>
#include <winsock.h>

#else

#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#endif

#include "../common/config.h"
#include "../common/utils.h"
#include "../common/messages.h"

#include "clientsockets.h"

// ---------------------------------------------------------- //
// variables


// socket stuff
SOCKADDR_IN		laddress;
int				addrsize = sizeof(SOCKADDR_IN);

#ifdef _WIN32
WSADATA			wsadata;		// needed by wsastartup()
#endif

// list of servers we can connect to
server_t		*serverlist;
int				servercount;

// array of ports
int				portstates[65536];
long			portsleft;



// ---------------------------------------------------------- //
// functions



// -----										           -- //
// initalise_sockets()							 20. 04. 2001
//
// initializes the sockets
// --													----- //

void	initialise_sockets()
{
	int i;

#ifdef _WIN32
	WSAStartup( MAKEWORD( 2, 0 ), &wsadata );
#endif 

	// initialize one socket per server
	for (i=0; i<servercount; i++)
		serverlist[i].sock = socket( AF_INET, SOCK_STREAM, 0 );
}



// -----										           -- //
// shutdown_sockets()							 20. 04. 2001
//
// deinitializes the sockets
// --													----- //

void	shutdown_sockets()
{
	int i;

#ifdef _WIN32
	WSACleanup();
#endif 

	// close and shutdown sockets
	for (i=0; i<servercount; i++)
		if ( INVALID_SOCKET != serverlist[i].sock )
		{
			shutdown( serverlist[i].sock, 0 );
			closesocket( serverlist[i].sock );
		}
}



// -----										           -- //
// evaluate_message()							 20. 04. 2001
//
// parameters:
//
// SOCKET	sock		the socket to read the message from
// msg_header *header	pointer to the header for this message
//
// return values
//
// 1 if a portresponse was received 0 otherwise
//
// reads a message from a socket and calls functions
//
// 23. 4. 2001 Kolrabi			fixed obvious overflow bug
//
// --													----- //

int		evaluate_message( SOCKET sock, msg_header *header )
{
	msg_response	response;
	char			*buf;
	int				res = 0;

	if (!header) return 0;

	buf = (char*)malloc( header->msgsize );
	recv( sock, buf, header->msgsize, 0 );

	switch( header->msgtype )
	{
		case MSG_RESPONSE:

			memcpy( &response, buf, sizeof(response) );

			// save port state
			portstates[response.port] = response.portstate;

			if (response.portstate == PORT_OPEN)
				logprintf( LVL_VERBOSE, "received state OPEN for port %u.", response.port );

			res = 1;

			break;
		// NOTE: handle other message types here

		default:
			logprintf( LVL_WARNING, "received unknown message type: 0x%04X", header->msgtype );
	}

	free( buf );

	return res;
}



// -----										           -- //
// handle_connection()							 20. 04. 2001
//
// parameters:
//
// int		server		number of the server to handle
//
// receives a header and then calls evaluate_message
// --													----- //

void	handle_connection( int server )
{
	msg_header		header;
	int				res;

	// if not connected do nothing
	if (serverlist[server].sock == INVALID_SOCKET) return;

	res = recv( serverlist[server].sock, (char*)&header, sizeof(header), 0);

	// an error occured(?)
	if (res == SOCKET_ERROR || res == 0)
	{
		shutdown( serverlist[server].sock, 0 );
		closesocket( serverlist[server].sock );
		serverlist[server].sock = INVALID_SOCKET;
		return;
	}

	// if we received a port state then we need to check one port less
	if (evaluate_message( serverlist[server].sock, &header ))
		serverlist[server].portstocheck --;

	// if there are no ports left for this server
	if (!serverlist[server].portstocheck)
	{
		// close this socket
		shutdown( serverlist[server].sock, 0);
		closesocket( serverlist[server].sock );

		// mark as invalid
		serverlist[server].sock = INVALID_SOCKET;
		return;
	}
}



// -----										           -- //
// make_connection()   							 20. 04. 2001
//
// connects to server and sends request
//
// 22. 4. 2001 Kolrabi			Added randomize & delay
// --													----- //

void	make_connection( char *name, long beginport, long endport )
{
	msg_header	header;
	msg_request	request;
	long		i;
	long		server;
	long		currentport;
	int			connectedsockets = 0;
	HOSTENT		*he;
	int			res;

	for (i = 0; i<65536; i++)
		portstates[i] = PORT_UNKNOWN;

	// connect sockets
	for (i=0; i<servercount; i++)
	{
		// set address and port
		laddress.sin_family			= AF_INET;
		laddress.sin_port			= htons( (unsigned short)serverlist[i].port );
		laddress.sin_addr.s_addr	= inet_addr( serverlist[i].name );

		// connect
		res = connect( serverlist[i].sock, (struct sockaddr*)&laddress, addrsize );

		// didnt work?
		if ( res == SOCKET_ERROR )
		{
#ifdef _WIN32
			logprintf( LVL_WARNING, "could not connect to server %s port %u", serverlist[i].name, serverlist[i].port );
#else
			logprintf( LVL_WARNING, "could not connect to server %s port %u: %s", serverlist[i].name, serverlist[i].port, strerror(errno) );
#endif
			shutdown( serverlist[i].sock, 0 );
			closesocket( serverlist[i].sock );
			serverlist[i].sock = INVALID_SOCKET;
			continue;
		}

		// otherwise 
		connectedsockets++;
	}

	// if no socket was connected we cannot scan
	if (!connectedsockets)
		logprintf( LVL_CRITICAL, "could not establish any server connection" );

	// distribute ports to servers
	i		= endport - beginport;
	
	logprintf( LVL_MESSAGE, "distributing %u ports to %u server(s)", i, connectedsockets );

	server	= 0;
	while(i)
	{
		server++;
		server %= servercount;

		if ( serverlist[server].sock != INVALID_SOCKET)
		{
			serverlist[server].portstocheck++;
			i--;
		}
	}

	// resolve host to be scanned
	he = gethostbyname( name );

	if (!he)
		logprintf( LVL_ERROR, "could not resolve host %s", name );

	// send requests
	currentport = beginport;
	for (i=0; i<servercount; i++)
	{
		if ( serverlist[i].sock == INVALID_SOCKET)
			continue;

		sprintf( request.servername, "%s", inet_ntoa(*(struct in_addr*)(he->h_addr_list[0])) );
		
		if ( strcmp( name, request.servername ) )
			logprintf( LVL_MESSAGE, "resolved %s to %s", name, request.servername );

		request.firstport	= currentport;
		request.lastport	= currentport+serverlist[i].portstocheck;
		request.type		= scantype;
		request.delay		= scandelay;
		request.timeout		= porttimeout;
		request.randomize	= randomizescan;

		header.msgtype		= MSG_REQUEST;
		header.msgsize		= sizeof(request);
		header.version		= MSG_VERSION;

		res=send( serverlist[i].sock, (char*)&header, sizeof(header), 0);
		if(res == INVALID_SOCKET) {
			logprintf(LVL_ERROR, "could not send MSG_HEADER to %s", request.servername);
			serverlist[i].sock == INVALID_SOCKET;
			continue;
		}
		
		res=send( serverlist[i].sock, (char*)&request, sizeof(request), 0);
		if(res == INVALID_SOCKET) {
			logprintf(LVL_ERROR, "could not send MSG_REQUEST to %s", request.servername);
			serverlist[i].sock == INVALID_SOCKET;
			continue;
		}

		currentport += serverlist[i].portstocheck;
	}

	while(connectedsockets)
	{
		for (i=0; i<servercount; i++)
			if ( serverlist[i].sock != INVALID_SOCKET )
			{
				handle_connection(i);

				if ( serverlist[i].sock == INVALID_SOCKET ) connectedsockets--;
			}
	}
}



// -----										           -- //
// fill_server_list()							 20. 04. 2001
//
// fills the server_t array with 
// --													----- //

void	fill_server_list()
{
	char	*p,*p2;
	int		i;
	HOSTENT	*he;

	servercount = 1;

	p = servers;
	
	while( p = strchr(++p, ',')) servercount++;

	serverlist = malloc(servercount * sizeof(server_t));

	p = servers;

	for (i = 0; i<servercount; i++)
	{
		p2 = strchr(p, ':');
		if (p2) { *p2 = 0; serverlist[i].port = atoi(p2+1); } else serverlist[i].port = 23555;

		if (p2)	p2 = strchr(p2+1, ','); else p2 = strchr(p, ',');
		if (p2) *p2 = 0;

		p = strtrm(p, " \n\t");

		p = strlwr(p);
		he = gethostbyname( p );
		if (he)
			sprintf( serverlist[i].name, "%s", inet_ntoa(*(struct in_addr*)he->h_addr_list[0]) );
		else
		{
			if (!strcmp(p, "localhost"))
				sprintf( serverlist[i].name, "127.0.0.1", p );
			else
				sprintf( serverlist[i].name, "%s", p );
		}

		serverlist[i].sock = INVALID_SOCKET;
		serverlist[i].portstocheck = 0;
	
		p = p2+1;
	}

}
