/*  Copyright (C) 2001Bjrn Paetzel                     kolrabi@gmx.de
   
    Bjrn Paetzel
	Gelsenkircher Str. 66
	D-44649 Herne

  ------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation;  either  version 2 of the  License, or
    any later version.

    This program is distributed in the hope that it  will be useful, but
    WITHOUT  ANY  WARRANTY;  without   even  the  implied   warranty  of
	MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU
    General Public License for more details.
	
	You should have received a copy  of the  GNU  General Public License
    along with this program (GPL.TXT); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, 
	Boston, MA
    02111-1307, USA.
*/

// ---------------------------------------------------------- //
// main.c

// ---------------------------------------------------------- //
// includes

#include <stdlib.h>
#include <stdio.h>

#ifdef _WIN32

#include <windows.h>
#include <winsock.h>

#else

#include <sys/types.h>
#include <sys/socket.h>

#endif

#include "../common/messages.h"
#include "../common/config.h"
#include "../common/utils.h"

#include "clientsockets.h"

// ---------------------------------------------------------- //
// functions

// -----										           -- //
// usage()                     					 19. 04. 2001
//
// prints the syntax on the screen
// --													----- //

void	usage( char *name )
{
	printf("Usage: client [options] <host to scan>\n\n" );
	printf("Options are:\n\n");
	printf("  -r         dont scan ports sequentially but randomized\n");
	printf("  -txx       wait xx seconds between ports\n");

	printf("  -bxx       begin with port xx\n");
	printf("  -exx       end with port xx\n");

        printf("  -mxx       port timeout for stealthscans\n");
	printf("  -sx        Scan Type\n");
	printf("    0        Connect Scan \n");
	printf("    1        SYN Scan \n");
	printf("    2        FIN Scan\n");
	printf("    3        XMAS Scan\n");
	printf("    4        NULL Scan\n");
	printf("    5        ACK Scan\n");
	printf("    6        Ping Scan (unimplemented) \n");
	
	printf("\n");
}

// -----										           -- //
// main()				    					 19. 04. 2001
//
// parameters:
//
// standard main parameters
//
// return values:
//
// nonzero if scan was successful
// zero if failed
// --													----- //

int main(int argc, char *argv[])
{
	long	i;

	printf("Distributed Port Scanner Client Version %s - %s\n", DSCANVERSION, DSCANBUILDTIME);
	printf("Coded for U.N.F.: http://www.u-n-f.com\n\n");
	printf("Copyright (C) 2001 by Bjrn Paetzel (kolrabi@gmx.de)\n\n");

	if (argc==1)
	{
		usage( argv[0] );
		return 0;
	}

	parse_command_line( argc, argv );
	read_config_file( configfile );
	check_options();

	fill_server_list();
	initialise_sockets();

	for ( i = 1; i<argc; i++)
	{
		if ( *argv[i] != '-' )
		{
			make_connection( argv[i], beginport, endport );
	
			printf("\n\nFound following open ports on %s:\n", argv[i]);

			for (i=0; i<65536; i++) {
				if(portstates[i] == PORT_OPEN)
					printf("port %5u: %s\n", i, getportname(i) );
				
				if(portstates[i] == PORT_FILTERED)
					printf("filtered port %5u: %s\n", i, getportname(i) );

			}
		}
	}
	shutdown_sockets();

	printf("\n");

	return 1;
}
