/*  Copyright (C) 2001Bjrn Paetzel                     kolrabi@gmx.de
   
    Bjrn Paetzel
	Gelsenkircher Str. 66
	D-44649 Herne

  ------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation;  either  version 2 of the  License, or
    any later version.

    This program is distributed in the hope that it  will be useful, but
    WITHOUT  ANY  WARRANTY;  without   even  the  implied   warranty  of
	MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU
    General Public License for more details.
	
	You should have received a copy  of the  GNU  General Public License
    along with this program (GPL.TXT); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, 
	Boston, MA
    02111-1307, USA.
*/

// ---------------------------------------------------------- //
// utils.c

// ---------------------------------------------------------- //
// includes

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <stdarg.h>

#include "config.h"
#include "utils.h"

// ---------------------------------------------------------- //
// functions

#ifndef _WIN32

// here are some functions that are not ansi c

char	charupr( char c )
{
	if ( c>='a' && c<='z' )
		return c-32;

	return c;
}

char	charlwr( char c )
{
	if ( c>='A' && c<='Z' )
		return c+32;

	return c;
}

char	*strupr( char *string )
{
	char *p = string;

	while (*p)
	{
		*p = charupr(*p);
		p++;
	}

	return string;
}

char	*strlwr( char *string )
{
	char *p = string;

	while (*p)
	{
		*p = charlwr(*p);
		p++;
	}

	return string;
}

#endif

char	*strtrml( const char *string, char *trimchrs )
{
	int		l = strlen(string);
	char	*p;

	p = (char*)string;

	while(strchr( trimchrs, *p ) && p<(string+l)) p++;

	return p;
}

char	*strtrmr( const char *string, char *trimchrs )
{
	int		l = strlen(string);
	char	*p;

	p = (char*)string+l-1;

	while(strchr( trimchrs, *p ) && p>(string+1)) p--;

	if (p<string+l-1 && *p)
	{
		p++;
		*p = 0;
	}

	return (char*)string;
}



char	*strtrm( const char *string, char *trimchrs )
{
	char	*p = strtrml( string, trimchrs );

	p = strtrmr( string, trimchrs );
	p = strtrml( p, trimchrs );

	return p;
}




// ---------------------------------------------------------- //
// logging functions

char	*getlevelname( int level )
{
	switch (level)
	{
		case LVL_DEBUG:		return "DGB";
		case LVL_VERBOSE:	return "VER";
		case LVL_MESSAGE:	return "MSG";
		case LVL_WARNING:	return "WRN";
		case LVL_ERROR:		return "ERR";
		case LVL_CRITICAL:	return "CRT";
	}
	return "???";
}

void	logprintf( int level, char *fmt, ... )
{
	va_list argptr;
	char	text[1024];

	va_start (argptr, fmt);
	vsprintf (text, fmt, argptr);
	va_end (argptr);

	if (level>=debuglevel)
		printf("(%s) %s\n", getlevelname(level), text);

	if (level>2)
		exit(level);
}

char	*getportname( int port )
{
	switch(port)
	{
		case	1:		return "tcpmux";	
		case	2:		return "compressnet";	    
		case	3:		return "compressnet";		
		case	5:		return "echo";		
		case	7:		return "echo";		
		case	9:		return "discard";	

		case    11:		return "systat";	
		case	13:		return "daytime";	
		case	15:		return "netstat";	
		case	17:		return "qotd";		
		case	18:		return "msp";	
		case	19:		return "chargen";	

		case	20:		return "ftp-data";	
		case	21:		return "ftp";		
		case	22:		return "ssh";		
		case	23:		return "telnet";	
		case	24:		return "priv-mail";   	
		case	25:		return "smtp";   	
		case	27:		return "nsw-fe";   	
		case	29:		return "msg-icp";   	

		case	31:		return "msg-auth";   	
		case	33:		return "dsp";   	
		case	35:		return "priv-print";   	
		case	37:		return "timeserver";	
		case	38:		return "rap";   	
		case	39:		return "rlp";   	

		case	41:		return "graphics";   	
		case	42:		return "name";		
		case	43:		return "whois";		
		case	44:		return "mpm-flags";		
		case	45:		return "mpm";		
		case	46:		return "mpm-snd";		
		case	47:		return "ni-ftp";		
		case	48:		return "auditd";		
		case	49:		return "tacas";		

		case	50:		return "re-mail-ck";		
		case	51:		return "la-maint";		
		case	52:		return "xns-time";		
		case	53:		return "domain";	
		case	54:		return "xns-ch";	
		case	55:		return "isi-gl";	
		case	56:		return "xns-auth";	
		case	57:		return "mtp / priv-term";		
		case	58:		return "xns-mail";		
		case	59:		return "priv-file";		

		case	61:		return "ni-mail";		
		case	62:		return "acas";		
		case	63:		return "via-ftp";		
		case	64:		return "covia";		
		case	65:		return "tacas-ds";		
		case	66:		return "sql*net";		
		case	67:		return "bootp";		
		case	68:		return "bootpc";		
		case	69:		return "tftp";		

		case	70:		return "gopher";		
		case	71:		return "netrjs-1";		
		case	72:		return "netrjs-2";		
		case	73:		return "netrjs-3";		
		case	74:		return "netrjs-4";		
		case	75:		return "priv-dial";		
		case	76:		return "deos";		
		case	77:		return "rje";		
		case	78:		return "vettcp";	
		case	79:		return "finger";	

		case	80:		return "http";		
		case	81:		return "hosts2-n2";		
		case	82:		return "xfer";		
		case	83:		return "mit-ml-dev";		
		case	84:		return "ctf";		
		case	85:		return "mit-ml-dev";		
		case	86:		return "mfcobol";		
		case	87:		return "link";		
		case	88:		return "kerberos-sec";		
		case	89:		return "su-mit-tg";		

		case	90:		return "dnsix";		
		case	91:		return "mit-dov";		
		case	92:		return "npp";		
		case	93:		return "dcp";		
		case	94:		return "objcall";		
		case	95:		return "supdup";	
		case	96:		return "dixie";	
		case	97:		return "swift";	
		case	98:		return "linuxconf / tacnews";	
		case	99:		return "metagram";	

		case	100:	return "newacct";	
		case	101:	return "hostnames";	
		case	102:	return "iso-tsap";	
		case	103:	return "dictionary / x400 / gppnp";	
		case	104:	return "acr-nema / x400-snd";	
		case	105:	return "csnet-ns";	
		case	106:	return "pop3pw";	
		case	107:	return "rtelnet";	
		case	108:	return "snagas";	
		case	109:	return "pop / pop2";		

		case	110:	return "pop3";		
		case	111:	return "portmap / sunrpc";	
		case	112:	return "mcidas";	
		case	113:	return "auth";		
		case	114:	return "audionews";	
		case	115:	return "sftp";		
		case	116:	return "ansanotify";	
		case	117:	return "uucp-path";		
		case	118:	return "sqlserv";	
		case	119:	return "nntp";		

		case	120:	return "cfdpkt";	
		case	121:	return "erpc";	
		case	122:	return "smakynet";	
		case	123:	return "ntp";		
		case	124:	return "ansatrader";	
		case	125:	return "locus-map";	
		case	126:	return "unitary";	
		case	127:	return "locus-com";	
		case	128:	return "gss-xclient";	
		case	129:	return "pwdgen";	

		case	130:	return "cisco-fnative";	
		case	131:	return "cisco-fnative";	
		case	132:	return "cisco-sysmaint";	
		case	133:	return "statsrv";	
		case	134:	return "ingres";	
		case	135:	return "loc-srv";	
		case	136:	return "profile";	
		case	137:	return "netbios-name";		
		case	138:	return "netbios-datagram";		
		case	139:	return "netbios-session";		

		case	140:	return "emfis-data";	
		case	141:	return "emfis-ctnl";	
		case	142:	return "bl-idm";	
		case	143:	return "imap2";	
		case	144:	return "NeWS";		
		case	145:	return "usaac";	
		case	146:	return "iso-tp0";	
		case	147:	return "iso-ip";	
		case	148:	return "cronus";	
		case	149:	return "aed-512";	

		case	150:	return "sql-net";	
		case	151:	return "bems";	
		case	152:	return "bftp";	
		case	153:	return "sgmp";		
		case	154:	return "netsc-prod";	
		case	155:	return "netsc-dev";	
		case	156:	return "sqlserv";	
		case	157:	return "knet-cmp";	
		case	158:	return "tcprepo (PCMAIL)";		
		case	159:	return "nss-routing";	

		case	160:	return "sgmp-traps";	
		case	161:	return "snmp";		
		case	162:	return "snmp-trap";		
		case	163:	return "cmip-man";	
		case	164:	return "cmip-agent";	
		case	165:	return "xns-courier";	
		case	166:	return "s-net";	
		case	167:	return "namp";	
		case	168:	return "rsvd";	
		case	169:	return "send";	

		case	170:	return "print-srv";		
		case	171:	return "multiplex";	
		case	172:	return "cl-1";	
		case	173:	return "xyplex-mux";	
		case	174:	return "mailq";	
		case	175:	return "vmnet";		
		case	176:	return "genrad-mux";	
		case	177:	return "xdmcp";	
		case	178:	return "nextstep";	
		case	179:	return "bgp";	

		case	180:	return "ris";	
		case	181:	return "unify";	
		case	182:	return "audit";	
		case	183:	return "ocbinder";	
		case	184:	return "ocserver";	
		case	185:	return "remote-kis";	
		case	186:	return "kis";	
		case	187:	return "aci";	
		case	188:	return "mumps";	
		case	189:	return "qft";	
		case	190: return "gacp";
		case	191: return "prospero";
		case	192: return "osu-nms";
		case	193: return "srmp";
		case	194: return "irc";
		case	195: return "dn6-nlm-aud";
		case	196: return "dn6-smm-red";
		case	197: return "dls";
		case	198: return "dls-mon";
		case	199: return "smux";
		case	200: return "src";
		case	201: return "at-rtmp";
		case	202: return "at-nbp";
		case	203: return "at-3";
		case	204: return "at-echo";
		case	205: return "at-5";
		case	206: return "at-zis";
		case	207: return "at-7";
		case	208: return "at-8";
		case	209: return "tam";
		case	210: return "z39.50";
		case	211: return "914c-g";
		case	212: return "anet";
		case	213: return "ipx";
		case	214: return "vmpwscs";
		case	215: return "softpc";
		case	216: return "atls";
		case	217: return "dbase";
		case	218: return "mpp";
		case	219: return "uarps";
		case	220: return "imap3";
		case	221: return "fln-spx";
		case	222: return "rsh-spx";
		case	223: return "cdc";
		case	242: return "direct";
		case	243: return "sur-meas";
		case	244: return "dayna";
		case	245: return "link";
		case	246: return "dsp3270";
		case	247: return "subntbcst_tftp";
		case	248: return "bhfhs";
		case	256: return "rap";
		case	257: return "set";
		case	258: return "yak-chat";
		case	259: return "esro-gen";
		case	260: return "openport";
		case	261: return "nsiiops";
		case	262: return "arcisdms";
		case	263: return "hdap";
		case	264: return "bgmp";
		case	280: return "http-mgmt";
		case	281: return "personal-link";
		case	282: return "cableport-ax";
		case	308: return "novastorbakcup";
		case	309: return "entrusttime";
		case	310: return "bhmds";
		case	311: return "asip-webadmin";
		case	312: return "vslmp";
		case	313: return "magenta-logic";
		case	314: return "opalis-robot";
		case	315: return "dpsi";
		case	316: return "decauth";
		case	317: return "zannet";
		case	321: return "pip";
		case	344: return "pdap";
		case	345: return "pawserv";
		case	346: return "zserv";
		case	347: return "fatserv";
		case	348: return "csi-sgwp";
		case	349: return "mftp";
		case	350: return "matip-type-a";
		case	351: return "matip-type-b";
		case	352: return "dtag-ste-sb";
		case	353: return "ndsauth";
		case	354: return "bh611";
		case	355: return "datex-asn";
		case	356: return "cloanto-net-1";
		case	357: return "bhevent";
		case	358: return "shrinkwrap";
		case	359: return "tenebris_nts";
		case	360: return "scoi2odialog";
		case	361: return "semantix";
		case	362: return "srssend";
		case	363: return "rsvp_tunnel";
		case	364: return "aurora-cmgr";
		case	365: return "dtk";
		case	366: return "odmr";
		case	367: return "mortgageware";
		case	368: return "qbikgdp";
		case	369: return "rpc2portmap";
		case	370: return "codaauth2";
		case	371: return "clearcase";
		case	372: return "ulistserv";
		case	373: return "legent-1";
		case	374: return "legent-2";
		case	375: return "hassle";
		case	376: return "nip";
		case	377: return "tnETOS";
		case	378: return "dsETOS";
		case	379: return "is99c";
		case	380: return "is99s";
		case	381: return "hp-collector";
		case	382: return "hp-managed-node";
		case	383: return "hp-alarm-mgr";
		case	384: return "arns";
		case	385: return "ibm-app";
		case	386: return "asa";
		case	387: return "aurp";
		case	388: return "unidata-ldm";
		case	389: return "ldap";
		case	390: return "uis";
		case	391: return "synotics-relay";
		case	392: return "synotics-broker";
		case	393: return "dis";
		case	394: return "embl-ndt";
		case	395: return "netcp";
		case	396: return "netware-ip";
		case	397: return "mptn";
		case	398: return "kryptolan";
		case	399: return "iso-tsap-c2";
		case	400: return "work-sol";
		case	401: return "ups";
		case	402: return "genie";
		case	403: return "decap";
		case	404: return "nced";
		case	405: return "ncld";
		case	406: return "imsp";
		case	407: return "timbuktu";
		case	408: return "prm-sm";
		case	409: return "prm-nm";
		case	410: return "decladebug";
		case	411: return "rmt";
		case	412: return "synoptics-trap";
		case	413: return "smsp";
		case	414: return "infoseek";
		case	415: return "bnet";
		case	416: return "silverplatter";
		case	417: return "onmux";
		case	418: return "hyper-g";
		case	419: return "ariel1";
		case	420: return "smpte";
		case	421: return "ariel2";
		case	422: return "ariel3";
		case	423: return "opc-job-start";
		case	424: return "opc-job-track";
		case	425: return "icad-el";
		case	426: return "smartsdp";
		case	427: return "svrloc";
		case	428: return "ocs_cmu";
		case	429: return "ocs_amu";
		case	430: return "utmpsd";
		case	431: return "utmpcd";
		case	432: return "iasd";
		case	433: return "nnsp";
		case	434: return "mobileip-agent";
		case	435: return "mobilip-mn";
		case	436: return "dna-cml";
		case	437: return "comscm";
		case	438: return "dsfgw";
		case	439: return "dasp";
		case	440: return "sgcp";
		case	441: return "decvms-sysmgt";
		case	442: return "cvc_hostd";
		case	443: return "https";
		case	444: return "snpp";
		case	445: return "microsoft-ds";
		case	446: return "ddm-rdb";
		case	447: return "ddm-dfm";
		case	448: return "ddm-ssl";
		case	449: return "as-servermap";
		case	450: return "tserver";
		case	451: return "sfs-smp-net";
		case	452: return "sfs-config";
		case	453: return "creativeserver";
		case	454: return "contentserver";
		case	455: return "creativepartnr";
		case	456: return "macon-tcp";
		case	457: return "scohelp";
		case	458: return "appleqtc";
		case	459: return "ampr-rcmd";
		case	460: return "skronk";
		case	461: return "datasurfsrv";
		case	462: return "datasurfsrvsec";
		case	463: return "alpes";
		case	464: return "kpasswd5";
		case	465: return "smtps";
		case	466: return "digital-vrc";
		case	467: return "mylex-mapd";
		case	468: return "photuris";
		case	469: return "rcp";
		case	470: return "scx-proxy";
		case	471: return "mondex";
		case	472: return "ljk-login";
		case	473: return "hybrid-pop";
		case	474: return "tn-tl-w1";
		case	475: return "tcpnethaspsrv";
		case	476: return "tn-tl-fd1";
		case	477: return "ss7ns";
		case	478: return "spsc";
		case	479: return "iafserver";
		case	480: return "loadsrv";
		case	481: return "dvs";
		case	482: return "bgs-nsi";
		case	483: return "ulpnet";
		case	484: return "integra-sme";
		case	485: return "powerburst";
		case	486: return "sstats";
		case	487: return "saft";
		case	488: return "gss-http";
		case	489: return "nest-protocol";
		case	490: return "micom-pfs";
		case	491: return "go-login";
		case	492: return "ticf-1";
		case	493: return "ticf-2";
		case	494: return "pov-ray";
		case	495: return "intecourier";
		case	496: return "pim-rp-disc";
		case	497: return "dantz";
		case	498: return "siam";
		case	499: return "iso-ill";
		case	500: return "isakmp";
		case	501: return "stmf";
		case	502: return "asa-appl-proto";
		case	503: return "intrinsa";
		case	504: return "citadel";
		case	505: return "mailbox-lm";
		case	506: return "ohimsrv";
		case	507: return "crs";
		case	508: return "xvttp";
		case	509: return "snare";
		case	510: return "fcp";
		case	511: return "passgo";
		case	512: return "exec";
		case	513: return "login";
		case	514: return "shell";
		case	515: return "printer";
		case	516: return "videotex";
		case	517: return "talk";
		case	518: return "ntalk";
		case	519: return "utime";
		case	520: return "efs";
		case	521: return "ripng";
		case	522: return "ulp";
		case	523: return "ibm-db2";
		case	524: return "ncp";
		case	525: return "timed";
		case	526: return "tempo";
		case	527: return "stx";
		case	528: return "custix";
		case	529: return "irc-serv";
		case	530: return "courier";
		case	531: return "conference";
		case	532: return "netnews";
		case	533: return "netwall";
		case	534: return "mm-admin";
		case	535: return "iiop";
		case	536: return "opalis-rdv";
		case	537: return "nmsp";
		case	538: return "gdomap";
		case	539: return "apertus-ldp";
		case	540: return "uucp";
		case	541: return "uucp-rlogin";
		case	542: return "commerce";
		case	543: return "klogin";
		case	544: return "kshell";
		case	545: return "ekshell";
		case	546: return "dhcpv6-client";
		case	547: return "dhcpv6-server";
		case	548: return "afpovertcp";
		case	549: return "idfp";
		case	550: return "new-rwho";
		case	551: return "cybercash";
		case	552: return "deviceshare";
		case	553: return "pirp";
		case	554: return "rtsp";
		case	555: return "dsf";
		case	556: return "remotefs";
		case	557: return "openvms-sysipc";
		case	558: return "sdnskmp";
		case	559: return "teedtap";
		case	560: return "rmonitor";
		case	561: return "monitor";
		case	562: return "chshell";
		case	563: return "snews";
		case	564: return "9pfs";
		case	565: return "whoami";
		case	566: return "streettalk";
		case	567: return "banyan-rpc";
		case	568: return "ms-shuttle";
		case	569: return "ms-rome";
		case	570: return "meter";
		case	571: return "umeter";
		case	572: return "sonar";
		case	573: return "banyan-vip";
		case	574: return "ftp-agent";
		case	575: return "vemmi";
		case	576: return "ipcd";
		case	577: return "vnas";
		case	578: return "ipdd";
		case	579: return "decbsrv";
		case	580: return "sntp-heartbeat";
		case	581: return "bdp";
		case	582: return "scc-security";
		case	583: return "philips-vc";
		case	584: return "keyserver";
		case	585: return "imap4-ssl";
		case	586: return "password-chg";
		case	587: return "submission";
		case	588: return "cal";
		case	589: return "eyelink";
		case	590: return "tns-cml";
		case	591: return "http-alt";
		case	592: return "eudora-set";
		case	593: return "http-rpc-epmap";
		case	594: return "tpip";
		case	595: return "cab-protocol";
		case	596: return "smsd";
		case	597: return "ptcnameservice";
		case	598: return "sco-websrvrmg3";
		case	599: return "acp";
		case	600: return "ipcserver";
		case	606: return "urm";
		case	607: return "nqs";
		case	608: return "sift-uft";
		case	609: return "npmp-trap";
		case	610: return "npmp-local";
		case	611: return "npmp-gui";
		case	628: return "qmqp";
		case	634: return "ginad";
		case	636: return "ldapssl";
		case	666: return "doom";
		case	704: return "elcsd";
		case	709: return "entrustmanager";
		case	729: return "netviewdm1";
		case	730: return "netviewdm2";
		case	731: return "netviewdm3";
		case	740: return "netcp";
		case	741: return "netgw";
		case	742: return "netrcs";
		case	744: return "flexlm";
		case	747: return "fujitsu-dev";
		case	748: return "ris-cm";
		case	749: return "kerberos-adm";
		case	750: return "kerberos";
		case	751: return "kerberos_master";
		case	752: return "qrh";
		case	753: return "rrh";
		case	754: return "krb_prop";
		case	758: return "nlogin";
		case	759: return "con";
		case	760: return "krbupdate";
		case	761: return "kpasswd";
		case	762: return "quotad";
		case	763: return "cycleserv";
		case	764: return "omserv";
		case	765: return "webster";
		case	767: return "phonebook";
		case	769: return "vid";
		case	770: return "cadlock";
		case	771: return "rtip";
		case	772: return "cycleserv2";
		case	773: return "submit";
		case	774: return "rpasswd";
		case	775: return "entomb";
		case	776: return "wpages";
		case	780: return "wpgs";
		case	781: return "hp-collector";
		case	782: return "hp-managed-node";
		case	783: return "hp-alarm-mgr";
		case	786: return "concert";
		case	799: return "controlit";
		case	800: return "mdbs_daemon";
		case	801: return "device";
		case	871: return "supfilesrv";
		case	888: return "accessbuilder";
		case	989: return "ftps-data";
		case	901: return "samba-swat";
		case	990: return "ftps";
		case	992: return "telnets";
		case	993: return "imaps";
		case	994: return "ircs";
		case	995: return "pop3s";
		case	996: return "xtreelic";
		case	997: return "maitrd";
		case	998: return "busboy";
		case	999: return "garcon";
		case	1000: return "cadlock";
		case	1008: return "ufsd";
		case	1024: return "kdm";
		case	1025: return "listen";
		case	1026: return "nterm";
		case	1030: return "iad1";
		case	1031: return "iad2";
		case	1032: return "iad3";
		case	1058: return "nim";
		case	1059: return "nimreg";
		case	1067: return "instl_boots";
		case	1068: return "instl_bootc";
		case	1080: return "socks";
		case	1083: return "ansoft-lm-1";
		case	1084: return "ansoft-lm-2";
		case	1103: return "xaudio";
		case	1109: return "kpop";
		case	1110: return "nfsd-status";
		case	1112: return "msql";
		case	1127: return "supfiledbg";
		case	1155: return "nfa";
		case	1178: return "skkserv";
		case	1212: return "lupa";
		case	1222: return "nerv";
		case	1234: return "hotline";
		case	1241: return "msg";
		case	1248: return "hermes";
		case	1346: return "alta-ana-lm";
		case	1347: return "bbn-mmc";
		case	1348: return "bbn-mmx";
		case	1349: return "sbook";
		case	1350: return "editbench";
		case	1351: return "equationbuilder";
		case	1352: return "lotusnotes";
		case	1353: return "relief";
		case	1354: return "rightbrain";
		case	1355: return "intuitive-edge";
		case	1356: return "cuillamartin";
		case	1357: return "pegboard";
		case	1358: return "connlcli";
		case	1359: return "ftsrv";
		case	1360: return "mimer";
		case	1361: return "linx";
		case	1362: return "timeflies";
		case	1363: return "ndm-requester";
		case	1364: return "ndm-server";
		case	1365: return "adapt-sna";
		case	1366: return "netware-csp";
		case	1367: return "dcs";
		case	1368: return "screencast";
		case	1369: return "gv-us";
		case	1370: return "us-gv";
		case	1371: return "fc-cli";
		case	1372: return "fc-ser";
		case	1373: return "chromagrafx";
		case	1374: return "molly";
		case	1375: return "bytex";
		case	1376: return "ibm-pps";
		case	1377: return "cichlid";
		case	1378: return "elan";
		case	1379: return "dbreporter";
		case	1380: return "telesis-licman";
		case	1381: return "apple-licman";
		case	1383: return "gwha";
		case	1384: return "os-licman";
		case	1385: return "atex_elmd";
		case	1386: return "checksum";
		case	1387: return "cadsi-lm";
		case	1388: return "objective-dbc";
		case	1389: return "iclpv-dm";
		case	1390: return "iclpv-sc";
		case	1391: return "iclpv-sas";
		case	1392: return "iclpv-pm";
		case	1393: return "iclpv-nls";
		case	1394: return "iclpv-nlc";
		case	1395: return "iclpv-wsm";
		case	1396: return "dvl-activemail";
		case	1397: return "audio-activmail";
		case	1398: return "video-activmail";
		case	1399: return "cadkey-licman";
		case	1400: return "cadkey-tablet";
		case	1401: return "goldleaf-licman";
		case	1402: return "prm-sm-np";
		case	1403: return "prm-nm-np";
		case	1404: return "igi-lm";
		case	1405: return "ibm-res";
		case	1406: return "netlabs-lm";
		case	1407: return "dbsa-lm";
		case	1408: return "sophia-lm";
		case	1409: return "here-lm";
		case	1410: return "hiq";
		case	1411: return "af";
		case	1412: return "innosys";
		case	1413: return "innosys-acl";
		case	1414: return "ibm-mqseries";
		case	1415: return "dbstar";
		case	1416: return "novell-lu6.2";
		case	1417: return "timbuktu-srv1";
		case	1418: return "timbuktu-srv2";
		case	1419: return "timbuktu-srv3";
		case	1420: return "timbuktu-srv4";
		case	1421: return "gandalf-lm";
		case	1422: return "autodesk-lm";
		case	1423: return "essbase";
		case	1424: return "hybrid";
		case	1425: return "zion-lm";
		case	1426: return "sas-1";
		case	1427: return "mloadd";
		case	1428: return "informatik-lm";
		case	1429: return "nms";
		case	1430: return "tpdu";
		case	1431: return "rgtp";
		case	1432: return "blueberry-lm";
		case	1433: return "ms-sql-s";
		case	1434: return "ms-sql-m";
		case	1435: return "ibm-cics";
		case	1436: return "sas-2";
		case	1437: return "tabula";
		case	1438: return "eicon-server";
		case	1439: return "eicon-x25";
		case	1440: return "eicon-slp";
		case	1441: return "cadis-1";
		case	1442: return "cadis-2";
		case	1443: return "ies-lm";
		case	1444: return "marcam-lm";
		case	1445: return "proxima-lm";
		case	1446: return "ora-lm";
		case	1447: return "apri-lm";
		case	1448: return "oc-lm";
		case	1449: return "peport";
		case	1450: return "dwf";
		case	1451: return "infoman";
		case	1452: return "gtegsc-lm";
		case	1453: return "genie-lm";
		case	1454: return "interhdl_elmd";
		case	1455: return "esl-lm";
		case	1456: return "dca";
		case	1457: return "valisys-lm";
		case	1458: return "nrcabq-lm";
		case	1459: return "proshare1";
		case	1460: return "proshare2";
		case	1461: return "ibm_wrless_lan";
		case	1462: return "world-lm";
		case	1463: return "nucleus";
		case	1464: return "msl_lmd";
		case	1465: return "pipes";
		case	1466: return "oceansoft-lm";
		case	1467: return "csdmbase";
		case	1468: return "csdm";
		case	1469: return "aal-lm";
		case	1470: return "uaiact";
		case	1471: return "csdmbase";
		case	1472: return "csdm";
		case	1473: return "openmath";
		case	1474: return "telefinder";
		case	1475: return "taligent-lm";
		case	1476: return "clvm-cfg";
		case	1477: return "ms-sna-server";
		case	1478: return "ms-sna-base";
		case	1479: return "dberegister";
		case	1480: return "pacerforum";
		case	1481: return "airs";
		case	1482: return "miteksys-lm";
		case	1483: return "afs";
		case	1484: return "confluent";
		case	1485: return "lansource";
		case	1486: return "nms_topo_serv";
		case	1487: return "localinfosrvr";
		case	1488: return "docstor";
		case	1489: return "dmdocbroker";
		case	1490: return "insitu-conf";
		case	1491: return "anynetgateway";
		case	1492: return "stone-design-1";
		case	1493: return "netmap_lm";
		case	1494: return "citrix-ica";
		case	1495: return "cvc";
		case	1496: return "liberty-lm";
		case	1497: return "rfx-lm";
		case	1498: return "watcom-sql";
		case	1499: return "fhc";
		case	1500: return "vlsi-lm";
		case	1501: return "sas-3";
		case	1502: return "shivadiscovery";
		case	1503: return "imtc-mcs";
		case	1504: return "evb-elm";
		case	1505: return "funkproxy";
		case	1506: return "utcd";
		case	1507: return "symplex";
		case	1508: return "diagmond";
		case	1509: return "robcad-lm";
		case	1510: return "mvx-lm";
		case	1511: return "3l-l1";
		case	1512: return "wins";
		case	1513: return "fujitsu-dtc";
		case	1514: return "fujitsu-dtcns";
		case	1515: return "ifor-protocol";
		case	1516: return "vpad";
		case	1517: return "vpac";
		case	1518: return "vpvd";
		case	1519: return "vpvc";
		case	1520: return "atm-zip-office";
		case	1521: return "ncube-lm";
		case	1522: return "rna-lm";
		case	1523: return "cichild-lm";
		case	1524: return "ingreslock";
		case	1525: return "orasrv";
		case	1526: return "pdap-np";
		case	1527: return "tlisrv";
		case	1528: return "mciautoreg";
		case	1529: return "support";
		case	1530: return "rap-service";
		case	1531: return "rap-listen";
		case	1532: return "miroconnect";
		case	1533: return "virtual-places";
		case	1534: return "micromuse-lm";
		case	1535: return "ampr-info";
		case	1536: return "ampr-inter";
		case	1537: return "sdsc-lm";
		case	1538: return "3ds-lm";
		case	1539: return "intellistor-lm";
		case	1540: return "rds";
		case	1541: return "rds2";
		case	1542: return "gridgen-elmd";
		case	1543: return "simba-cs";
		case	1544: return "aspeclmd";
		case	1545: return "vistium-share";
		case	1546: return "abbaccuray";
		case	1547: return "laplink";
		case	1548: return "axon-lm";
		case	1549: return "shivahose";
		case	1550: return "3m-image-lm";
		case	1551: return "hecmtl-db";
		case	1552: return "pciarray";
		case	1600: return "issd";
		case	1650: return "nkd";
		case	1651: return "shiva_confsrvr";
		case	1652: return "xnmp";
		case	1661: return "netview-aix-1";
		case	1662: return "netview-aix-2";
		case	1663: return "netview-aix-3";
		case	1664: return "netview-aix-4";
		case	1665: return "netview-aix-5";
		case	1666: return "netview-aix-6";
		case	1667: return "netview-aix-7";
		case	1668: return "netview-aix-8";
		case	1669: return "netview-aix-9";
		case	1670: return "netview-aix-10";
		case	1671: return "netview-aix-11";
		case	1672: return "netview-aix-12";
		case	1723: return "pptp";
		case	1827: return "pcm";
		case	1986: return "licensedaemon";
		case	1987: return "tr-rsrb-p1";
		case	1988: return "tr-rsrb-p2";
		case	1989: return "tr-rsrb-p3";
		case	1990: return "stun-p1";
		case	1991: return "stun-p2";
		case	1992: return "stun-p3";
		case	1993: return "snmp-tcp-port";
		case	1994: return "stun-port";
		case	1995: return "perf-port";
		case	1996: return "tr-rsrb-port";
		case	1997: return "gdp-port";
		case	1998: return "x25-svc-port";
		case	1999: return "tcp-id-port";
		case	2000: return "callbook";
		case	2001: return "dc";
		case	2002: return "globe";
		case	2003: return "cfingerd";
		case	2004: return "mailbox";
		case	2005: return "deslogin";
		case	2006: return "invokator";
		case	2007: return "dectalk";
		case	2008: return "conf";
		case	2009: return "news";
		case	2010: return "search";
		case	2011: return "raid-cc";
		case	2012: return "ttyinfo";
		case	2013: return "raid-am";
		case	2014: return "troff";
		case	2015: return "cypress";
		case	2016: return "bootserver";
		case	2017: return "cypress-stat";
		case	2018: return "terminaldb";
		case	2019: return "whosockami";
		case	2020: return "xinupageserver";
		case	2021: return "servexec";
		case	2022: return "down";
		case	2023: return "xinuexpansion3";
		case	2024: return "xinuexpansion4";
		case	2025: return "ellpack";
		case	2026: return "scrabble";
		case	2027: return "shadowserver";
		case	2028: return "submitserver";
		case	2030: return "device2";
		case	2032: return "blackboard";
		case	2033: return "glogger";
		case	2034: return "scoremgr";
		case	2035: return "imsldoc";
		case	2038: return "objectmanager";
		case	2040: return "lam";
		case	2041: return "interbase";
		case	2042: return "isis";
		case	2043: return "isis-bcast";
		case	2044: return "rimsl";
		case	2045: return "cdfunc";
		case	2046: return "sdfunc";
		case	2047: return "dls";
		case	2048: return "dls-monitor";
		case	2049: return "nfs";
		case	2064: return "distrib-netassholes";
		case	2065: return "dlsrpn";
		case	2067: return "dlswpn";
		case	2105: return "eklogin";
		case	2106: return "ekshell";
		case	2108: return "rkinit";
		case	2111: return "kx";
		case	2112: return "kip";
		case	2120: return "kauth";
		case	2201: return "ats";
		case	2232: return "ivs-video";
		case	2241: return "ivsd";
		case	2301: return "compaqdiag";
		case	2307: return "pehelp";
		case	2401: return "cvspserver";
		case	2430: return "venus";
		case	2431: return "venus-se";
		case	2432: return "codasrv";
		case	2433: return "codasrv-se";
		case	2500: return "rtsserv";
		case	2501: return "rtsclient";
		case	2564: return "hp-3000-telnet";
		case	2600: return "zebrasrv";
		case	2601: return "zebra";
		case	2602: return "ripd";
		case	2603: return "ripngd";
		case	2604: return "ospfd";
		case	2605: return "bgpd";
		case	2627: return "webster";
		case	2638: return "sybase";
		case	2766: return "listen";
		case	2784: return "www-dev";
		case	3000: return "ppp";
		case	3001: return "nessusd";
		case	3005: return "deslogin";
		case	3006: return "deslogind";
		case	3049: return "cfs";
		case	3064: return "distrib-net-proxy";
		case	3086: return "sj3";
		case	3128: return "squid-http";
		case	3141: return "vmodem";
		case	3264: return "ccmail";
		case	3306: return "mysql";
		case	3333: return "dec-notes";
		case	3389: return "msrdp";
		case	3421: return "bmap";
		case	3455: return "prsvp";
		case	3456: return "vat";
		case	3457: return "vat-control";
		case	3462: return "track";
		case	3900: return "udt_os";
		case	3984: return "mapper-nodemgr";
		case	3985: return "mapper-mapethd";
		case	3986: return "mapper-ws_ethd";
		case	4008: return "netcheque";
		case	4045: return "lockd";
		case	4132: return "nuts_dem";
		case	4133: return "nuts_bootp";
		case	4144: return "wincim";
		case	4321: return "rwhois";
		case	4333: return "msql";
		case	4343: return "unicall";
		case	4444: return "krb524";
		case	4500: return "sae-urn";
		case	4557: return "fax";
		case	4559: return "hylafax";
		case	4672: return "rfa";
		case	5000: return "fics";
		case	5001: return "commplex-link";
		case	5002: return "rfe";
		case	5010: return "telelpathstart";
		case	5011: return "telelpathattack";
		case	5050: return "mmcc";
		case	5145: return "rmonitor_secure";
		case	5190: return "aol";
		case	5191: return "aol-1";
		case	5192: return "aol-2";
		case	5193: return "aol-3";
		case	5232: return "sgi-dgl";
		case	5236: return "padl2sim";
		case	5300: return "hacl-hb";
		case	5301: return "hacl-gs";
		case	5302: return "hacl-cfg";
		case	5303: return "hacl-probe";
		case	5304: return "hacl-local";
		case	5305: return "hacl-test";
		case	5308: return "cfengine";
		case	5432: return "postgres";
		case	5510: return "secureidprop";
		case	5520: return "sdlog";
		case	5530: return "sdserv";
		case	5540: return "sdreport";
		case	5550: return "sdadmind";
		case	5631: return "pcanywheredata";
		case	5632: return "pcanywherestat";
		case	5680: return "canna";
		case	5713: return "proshareaudio";
		case	5714: return "prosharevideo";
		case	5715: return "prosharedata";
		case	5716: return "prosharerequest";
		case	5717: return "prosharenotify";
		case	5800: return "vnc";
		case	5801: return "vnc";
		case	5900: return "vnc";
		case	5901: return "vnc-1";
		case	5902: return "vnc-2";
		case	5977: return "ncd-pref-tcp";
		case	5978: return "ncd-diag-tcp";
		case	5979: return "ncd-conf-tcp";
		case	5997: return "ncd-pref";
		case	5998: return "ncd-diag";
		case	5999: return "ncd-conf";
		case	6000: return "X11";
		case	6001: return "X11:1";
		case	6002: return "X11:2";
		case	6003: return "X11:3";
		case	6004: return "X11:4";
		case	6005: return "X11:5";
		case	6006: return "X11:6";
		case	6007: return "X11:7";
		case	6008: return "X11:8";
		case	6009: return "X11:9";
		case	6110: return "softcm";
		case	6111: return "spc";
		case	6112: return "dtspc";
		case	6141: return "meta-corp";
		case	6142: return "aspentec-lm";
		case	6143: return "watershed-lm";
		case	6144: return "statsci1-lm";
		case	6145: return "statsci2-lm";
		case	6146: return "lonewolf-lm";
		case	6147: return "montage-lm";
		case	6148: return "ricardo-lm";
		case	6558: return "xdsxdm";
		case	6666: return "irc-serv";
		case	6667: return "irc";
		case	6668: return "irc";
		case	6969: return "acmsoda";
		case	7000: return "afs3-fileserver";
		case	7001: return "afs3-callback";
		case	7002: return "afs3-prserver";
		case	7003: return "afs3-vlserver";
		case	7004: return "afs3-kaserver";
		case	7005: return "afs3-volser";
		case	7006: return "afs3-errors";
		case	7007: return "afs3-bos";
		case	7008: return "afs3-update";
		case	7009: return "afs3-rmtsys";
		case	7010: return "ups-onlinet";
		case	7100: return "font-service";
		case	7200: return "fodms";
		case	7201: return "dlip";
		case	7326: return "icb";
		case	8080: return "http-proxy";
		case	8888: return "sun-answerbook";
		case	8892: return "seosload";
		case	9090: return "zeus-admin";
		case	9100: return "jetdirect";
		case	9535: return "man";
		case	9876: return "sd";

		case	10000:	return "bnews / rscs0";		break;
		case	10001:	return "queue / rscs1";		break;
		case	10002:	return "poker / rscs2";		break;
		case	10003:	return "gateway / rscs3";		break;
		case	10004:	return "remp / rscs4";		break;
		case	10005:	return "rscs5 / stel";		break;
		case	10006:	return "rscs6";		break;
		case	10007:	return "rscs7";		break;
		case	10008:	return "rscs8";		break;
		case	10009:	return "rscs9";		break;
		case	10010:	return "rscsa";		break;
		case	10011:	return "rscsb";		break;
		case	10012:	return "qmaster";		break;
			
		case	10082: return "amandaidx";
		case	10083: return "amidxtape";
		case	11371: return "pksd";
		case	12345: return "NetBus";
		case	12346: return "NetBus";
		case	17007: return "isode-dua";
		case	18000: return "biimenu";
		case	20005: return "btx";
		case	22273: return "wnn6";
		case	22289: return "wnn6_Cn";
		case	22305: return "wnn6_Kr";
		case	22321: return "wnn6_Tw";

		case	23555:	return "dscan-server";		break;
		
		case	26208: return "wnn6_DS";
		case	27665: return "Trinoo_Master";
		case	31337: return "Elite";
		case	32770: return "sometimes-rpc3";
		case	32771: return "sometimes-rpc5";
		case	32772: return "sometimes-rpc7";
		case	32773: return "sometimes-rpc9";
		case	32774: return "sometimes-rpc11";
		case	32775: return "sometimes-rpc13";
		case	32776: return "sometimes-rpc15";
		case	32777: return "sometimes-rpc17";
		case	32778: return "sometimes-rpc19";
		case	32779: return "sometimes-rpc21";
		case	32780: return "sometimes-rpc23";
		case	32786: return "sometimes-rpc25";
		case	32787: return "sometimes-rpc27";
		case	43188: return "reachout";
		case	47557: return "dbbrowse";
		case	65301: return "pcanywhere";
	}
	return "??";
}
