/*  Copyright (C) 2001Bjrn Paetzel                     kolrabi@gmx.de
   
    Bjrn Paetzel
	Gelsenkircher Str. 66
	D-44649 Herne

  ------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation;  either  version 2 of the  License, or
    any later version.

    This program is distributed in the hope that it  will be useful, but
    WITHOUT  ANY  WARRANTY;  without   even  the  implied   warranty  of
	MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU
    General Public License for more details.
	
	You should have received a copy  of the  GNU  General Public License
    along with this program (GPL.TXT); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, 
	Boston, MA
    02111-1307, USA.
*/

// ---------------------------------------------------------- //
// scan.c

// ---------------------------------------------------------- //
// includes

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#ifdef _WIN32

#include <windows.h>
#include <winsock.h>

#else

#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <sys/time.h>
#include <linux/tcp.h>
#include <linux/ip.h>

#endif

#include "../common/messages.h"
#include "../common/config.h"

#include "serversockets.h"
#include "scan.h"



/* 
 * ip header check
 * 
 * ripped 
*/
unsigned short in_cksum(unsigned short *ptr, int nbytes)
{
        register long           sum;            /* assumes long == 32 bits */
        u_short                 oddbyte;
        register u_short        answer;         /* assumes u_short == 16 bits */

        /*
         * Our algorithm is simple, using a 32-bit accumulator (sum),
         * we add sequential 16-bit words to it, and at the end, fold back
         * all the carry bits from the top 16 bits into the lower 16 bits.
         */

        sum = 0;
        while (nbytes > 1)  {
                sum += *ptr++;
                nbytes -= 2;
        }

                                /* mop up an odd byte, if necessary */
        if (nbytes == 1) {
                oddbyte = 0;            /* make sure top half is zero */
                *((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
                sum += oddbyte;
        }

        /*
         * Add back carry outs from top 16 bits to low 16 bits.
         */

        sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
        sum += (sum >> 16);                     /* add carry */
        answer = ~sum;          /* ones-complement, then truncate to 16 bits */
        return(answer);
}



/* -----
 * stealthscan()
 *
 * parameters:
 *   char *name             name of the server to scan (dot numbered format)
 *   unsigned int mode      witch mode to scan: syn, null, fin or xmas
 *   int speed              time to wait for a port timeout
 * 
 * return values:
 *   PORT_OPEN, PORT_CLOSED or -1 (for an error)
 * 
 * scans stealthy a host if a given port is open
 * --
 *
 * by Anthraxx, 05.05.2001
 * 
 *
 * Added ACK Scan: 10.05.2001
 * 
 * todo: fragmented scanning
 * 
*/

int stealthscan(char *name, unsigned short port, unsigned int mode, int speed)
{

   struct recv_tcp
   {
      struct iphdr ip;
      struct tcphdr tcp;
      unsigned char data[65535];
   }  recv_tcp;

   struct pseudo_header
   {
      unsigned int source_address;
      unsigned int dest_address;
      unsigned char placeholder;
      unsigned char protocol;
      unsigned short tcp_length;
      struct tcphdr tcp;
   } pseudo_header;

   struct sockaddr_in sin;
   static int add = 0;
   struct timeval tv;
   struct tcphdr send_tcp;
   unsigned int dst_addr;
   unsigned int src_addr;
   int sock;
   int bla;
   int err;
   fd_set readfs;

   add++;
   dst_addr=inet_addr(name);
   src_addr=inet_addr(localaddress); /* normally linux fills in ip, but.. */
   bzero(&send_tcp, sizeof(struct tcphdr));

   send_tcp.source = getpid() + add;
   send_tcp.dest = htons(port);
   send_tcp.seq = getpid() + add;
   send_tcp.doff = 5;
   send_tcp.window = htons(512);

   /* scan technics? */
   /* mode 0 = null (nothing to change)
           1 = syn
           2 = fin
           3 = xmas
	   4 = ack
   */
   switch(mode)
   {
      case 1: send_tcp.syn = 1;
              break;

      case 2: send_tcp.fin = 1;
              break;

      case 3: send_tcp.fin = 1;
              send_tcp.psh = 1;
              send_tcp.urg = 1;
              break;
	      

      case 4: send_tcp.ack = 1;
	      break;
   }

   pseudo_header.source_address = src_addr;
   pseudo_header.dest_address = dst_addr;
   pseudo_header.placeholder = 0;
   pseudo_header.protocol = IPPROTO_TCP;
   pseudo_header.tcp_length = htons(20);

   bcopy(&send_tcp, &pseudo_header.tcp, 20);
   send_tcp.check = in_cksum((unsigned short *)&pseudo_header, 32);

   sin.sin_family = AF_INET;
   sin.sin_port = htons(port);
   sin.sin_addr.s_addr = dst_addr;

   sock = socket(AF_INET, SOCK_RAW, IPPROTO_TCP);
   if(sock < 0)
   {
      printf("couldnt open socket\n");
      return(-1);
   }

   /* set timer and prepare for select() */
   tv.tv_sec=0;
   tv.tv_usec=(speed*1000);
   FD_ZERO(&readfs);
   FD_SET(sock, &readfs);


   /* send packet */
   err=sendto(sock, &send_tcp, 20, 0, (struct sockaddr *)&sin, sizeof(sin));
   if(err < 0) {
     printf("Error when sending packet to port %s", port);
     return(-1);
   }
	 
   /* read answer */
   while(1)
   {
      bla=select(sock+1, &readfs, NULL, NULL, &tv);
      if(bla) {
        err=read(sock, (struct recv_tcp *)&recv_tcp, 65535);
        if(err < 0) {
          printf("error while reading answer from port %s", port);
          close(sock);
          return(-1);
        }
      } else {
	/* port timed out */
        close(sock);
	  
        if(mode == 4)
		return(PORT_FILTERED);
	else
		return(PORT_CLOSED);
      }

      /* is this our packet? */
      if(recv_tcp.tcp.dest == (getpid() + add))
      {
         close(sock);

	 if(recv_tcp.tcp.rst == 1) {
	 	if(mode == 4)
			return(PORT_UNFILTERED);
		else	
           		return(PORT_CLOSED);
	 } else {
           	return PORT_OPEN;
	 }
      }
   }

   return(PORT_CLOSED);
}


int		connectscan( char *name, int port )
{
	SOCKET			scansocket;
	SOCKADDR_IN		saddress;
	int				addrsize = sizeof(SOCKADDR_IN);

	// set address and port
	saddress.sin_family			= AF_INET;
	saddress.sin_port			= htons( port );
	saddress.sin_addr.s_addr	= inet_addr( name );

	scansocket = socket( AF_INET, SOCK_STREAM, 0 );

	if (!connect(scansocket, (struct sockaddr*)&saddress, addrsize ))
	{
		shutdown(scansocket,0);
		return PORT_OPEN;
	}

	shutdown(scansocket,0);
	return PORT_CLOSED;
}


// -----                                                                   -- //
// scan()                                                           21. 04. 2001
//
// parameters:
//
// char  	*name		name of the server to scan
//				this must be a dotted number string
// int		port		port number to scan
// int		type		type of scan to do
// int		timeout		port timeout
//
// return values
//
// PORT_OPEN or PORT_CLOSED or PORT_UNKNOWN
//
// scans a host if a given port is open
// --                                                                   ----- //

int		scan( char *name, int port, int type, int timeout )
{

        printf(".");
	
	switch(type)
	{
		case	SCAN_CONNECT:
			return connectscan( name, port );
			break;

		case	SCAN_SYN:
			return stealthscan(name, port, 1, timeout);
			break;

		case	SCAN_FIN:
			return stealthscan(name, port, 2, timeout);
			break;
			
		case	SCAN_XMAS:
			return stealthscan(name, port, 3, timeout);
			break;
			
		case	SCAN_NULL:
			return stealthscan(name, port, 0, timeout);
			break;
			
		case	SCAN_ACK:
			return stealthscan(name, port, 4, timeout);
			break;
	}
	
	return PORT_CLOSED;
}
