/*  Copyright (C) 2001Bjrn Paetzel                     kolrabi@gmx.de
   
    Bjrn Paetzel
	Gelsenkircher Str. 66
	D-44649 Herne

  ------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation;  either  version 2 of the  License, or
    any later version.

    This program is distributed in the hope that it  will be useful, but
    WITHOUT  ANY  WARRANTY;  without   even  the  implied   warranty  of
	MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU
    General Public License for more details.
	
	You should have received a copy  of the  GNU  General Public License
    along with this program (GPL.TXT); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, 
	Boston, MA
    02111-1307, USA.
*/

// ---------------------------------------------------------- //
// serversockets.c

// ---------------------------------------------------------- //
// includes

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/wait.h>

#ifdef _WIN32
#include <windows.h>
#include <winsock.h>
#else

#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <unistd.h>


#define	closesocket(a) close(a)

#endif

#include "../common/config.h"
#include "../common/utils.h"
#include "../common/messages.h"

#include "scan.h"

#include "serversockets.h"

// ---------------------------------------------------------- //
// variables


SOCKET			listensocket = INVALID_SOCKET;
SOCKADDR_IN		laddress;
int				addrsize = sizeof(SOCKADDR_IN);

#ifdef _WIN32

WSADATA		wsadata;

#endif





// ---------------------------------------------------------- //
// functions




// -----										           -- //
// initalise_sockets()							 19. 04. 2001
//
// initializes the server sockets
// --													----- //

void	initialise_sockets()
{

#ifdef _WIN32
	WSAStartup( MAKEWORD( 2, 0 ), &wsadata );
#endif 

	listensocket = socket( PF_INET, SOCK_STREAM, 0 );

	printf("socket = %d\n", listensocket );

	if ( listensocket == INVALID_SOCKET) 
	{ 
		logprintf( LVL_CRITICAL, "could not create socket.." ); 
		return; 
	}

	// set local address and port
	laddress.sin_family			= AF_INET;
    laddress.sin_port			= htons( listenport );
	laddress.sin_addr.s_addr	= INADDR_ANY;

	if ( SOCKET_ERROR == bind( listensocket, (SOCKADDR*)&laddress, sizeof(laddress)) )
	{
		logprintf( LVL_CRITICAL, "could not bind().." ); 
		return; 
	}

	// start listening

	if ( SOCKET_ERROR == listen( listensocket, 3 ) )
	{
		logprintf( LVL_CRITICAL, "could not start listening on port %u..", listenport ); 
		return; 
	}

	logprintf( LVL_MESSAGE, "now listening on port %u..", listenport ); 
}




// -----										           -- //
// shutdown_sockets()							 19. 04. 2001
//
// initializes the server sockets
// --													----- //

void	shutdown_sockets()
{

#ifdef _WIN32
	WSACleanup();
#endif 

}


#define		MAX(a,b) a>b?a:b

// -----										           -- //
// evaluate_message()							 20. 04. 2001
//
// parameters:
//
// SOCKET	sock		the socket to read the message from
// msg_header *header	pointer to the header for this message
//
// reads a message from a socket and calls functions
// --													----- //

void	evaluate_message( SOCKET sock, msg_header *header )
{
	char			*buf;
	msg_response	response;
	msg_request		request;
	msg_header		header2;

	long			i;
	int				res;

	long			portstoscan;
	int				scanned[65536];

	if (!header) return;

	buf = (char*)malloc( header->msgsize );
	res = recv( sock, buf, header->msgsize, 0 );

	if ( res == SOCKET_ERROR || res == 0)
	{
		printf("\n");
		logprintf( LVL_WARNING, "lost connection to client" );
		return;
	}

	switch( header->msgtype )
	{
		case MSG_REQUEST:

			memcpy( &request, buf, sizeof(request) );

			header2.msgtype = MSG_RESPONSE;
			header2.version = MSG_VERSION;
			header2.msgsize = sizeof(response);

			logprintf( LVL_MESSAGE, "received request for %s ports %u through %u", request.servername, request.firstport, request.lastport );

			portstoscan = request.lastport - request.firstport;
			for (i = 0; i<65536; i++)
				scanned[i] = 0;

			i = request.firstport;

			while(portstoscan)
			{
				if (request.randomize)
				{
					while(scanned[i])
						i = request.firstport + rand()%(request.lastport - request.firstport );
				}
				else i++;

#ifdef _WIN32
				Sleep( request.delay * 1000 );
#else
				sleep( request.delay );
#endif

				response.port = i;
				response.portstate = scan( request.servername, i, request.type, request.timeout );

				send( sock, (char*)&header2, sizeof(header2), 0 );
				res = send( sock, (char*)&response, sizeof(response), 0 );

				if (res == SOCKET_ERROR)
				{
					printf("\n");
					logprintf( LVL_WARNING, "lost connection to client" );
					break;
				}

				portstoscan--;
				scanned[i] = 1;
			}

			break;
		// NOTE: handle other message types here

		default:
			logprintf( LVL_WARNING, "received unknown message type: 0x%04X", header->msgtype );
	}

	free(buf);
	printf("\n");
}




// -----										           -- //
// handle_connection()							 20. 04. 2001
//
// parameters:
//
// SOCKET	sock		the socket to read msgs from
//
// handles the complete connection process
// --													----- //

void	handle_connection( SOCKET sock )
{
	msg_header		header;
	int				res = 1;

	if (sock == INVALID_SOCKET) return;

	while (1)
	{
		res = recv( sock, (char*)&header, sizeof(header), 0 );

		if ( res == SOCKET_ERROR )
		{
			logprintf( LVL_MESSAGE, "lost client connection" );
			break;
		}

		if ( res == 0 )
		{
			logprintf( LVL_MESSAGE, "connection closed" );
			break;
		}

		evaluate_message( sock, &header );
	}

}




// -----										           -- //
// server_loop()     							 19. 04. 2001
//
// accepts connections and calls the scan function
// --													----- //

void	server_loop()
{
	SOCKET			connectionsocket;
	SOCKADDR_IN		clientaddr;

	logprintf( LVL_DEBUG, "entering server_loop()");

	while(1)
	{
		connectionsocket = accept( listensocket, (struct sockaddr*)&clientaddr, &addrsize );
		if(connectionsocket >= 0) {
			logprintf( LVL_MESSAGE, "incoming connection from: %s", inet_ntoa( clientaddr.sin_addr ) ); 

			if(fork() == 0) {
				handle_connection( connectionsocket );

				shutdown( connectionsocket,0  );
				closesocket( connectionsocket );
				exit(0);
			}
		} else {
			logprintf( LVL_ERROR, "socket error");
		}
		connectionsocket = INVALID_SOCKET;

		/* children handling */
		/* i know, it's bad style, but i don't want all the code 
		   for signal'ing shit */ 
		waitpid(-1, NULL, WNOHANG);
	}
}
